<?php
namespace Sistemapay\Plazox\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use Psr\Log\LoggerInterface;
use Magento\Framework\Filesystem;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Message\ManagerInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Config\Model\ResourceModel\Config;
use Magento\Framework\App\ResourceConnection;


class SystemConfigSave implements ObserverInterface
{
    protected $logger;
    protected $filesystem;
    protected $scopeConfig;
    protected $messageManager;
    protected $storeManager;
    protected $configResource;
    protected $resource;

    public function __construct(
        LoggerInterface $logger,
        Filesystem $filesystem,
        ScopeConfigInterface $scopeConfig,
        ManagerInterface $messageManager,
        StoreManagerInterface $storeManager,
        Config $configResource,
        ResourceConnection $resource
    ) {
        $this->logger = $logger;
        $this->filesystem = $filesystem;
        $this->scopeConfig = $scopeConfig;
        $this->messageManager = $messageManager;
        $this->storeManager = $storeManager;
        $this->configResource = $configResource;
        $this->resource = $resource;
    }

    public function execute(Observer $observer)
    {
        $fileName='plazox.js';

        // Contenido del archivo JavaScript
        // Set Variables
        $jsContent = "let fonts_path='/".\Magento\Framework\App\Filesystem\DirectoryList::MEDIA . '/plazox/fonts'."';";
        $jsContent .= PHP_EOL."let magento_theme='".$this->scopeConfig->getValue('plazox/general/magento_theme')."';";
        $jsContent .= PHP_EOL."let bar_top_menu_show='".$this->scopeConfig->getValue('plazox/bar_top_menu/show_bar_top_menu')."';";
        $jsContent .= PHP_EOL."let bar_top_menu_show_icon='".$this->scopeConfig->getValue('plazox/bar_top_menu/show_icon')."';";
        $jsContent .= PHP_EOL."let bar_top_menu_text_message='".$this->scopeConfig->getValue('plazox/bar_top_menu/text_message')."';";
        $jsContent .= PHP_EOL."let bar_top_menu_style='".$this->scopeConfig->getValue('plazox/bar_top_menu/style')."';";
        $jsContent .= PHP_EOL."let product_page_show='".$this->scopeConfig->getValue('plazox/product_page/show_product_page')."';";
        $jsContent .= PHP_EOL."let product_page_show_banner='".$this->scopeConfig->getValue('plazox/product_page/show_banner')."';";
        $jsContent .= PHP_EOL."let floating_banner_visible='".$this->scopeConfig->getValue('plazox/floating_banner/visible')."';";
        $jsContent .= PHP_EOL."let floating_banner_position='".$this->scopeConfig->getValue('plazox/floating_banner/position')."';";
        $jsContent .= PHP_EOL."let floating_banner_design='".$this->scopeConfig->getValue('plazox/floating_banner/design')."';";
        $jsContent .= PHP_EOL;

        // Incluye traducciones
        $translationJsContent = $this->getTranslationsFileContent();
        $jsContent.= PHP_EOL . $translationJsContent;

        // Incluye contenido base
        $baseJsContent = $this->getBaseFileContent();
        $jsContent.= PHP_EOL . $baseJsContent; // Agregar una nueva línea*/


        // Guarda la url del archivo JS
        $this->saveConfigValue('plazox/config/fileurl', $fileName, 'default', 0);

        // Ruta completa para el archivo JavaScript en pub/media
        $mediaDirectory = $this->filesystem->getDirectoryWrite(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA);
        $filePath = $mediaDirectory->getAbsolutePath('plazox/'.$fileName);

        // Guarda el contenido en el archivo JavaScript
        $mediaDirectory->writeFile('plazox/'.$fileName, $jsContent);

        // Copia las fuentes a Media
        $this->copyFonts();

        //Website & StoreView
        if($observer->getStore() || $observer->getWebsite()){

            if($observer->getStore()) {
                $scope=\Magento\Store\Model\ScopeInterface::SCOPE_STORE;
                $fileName = 'store/' . $observer->getStore() . '/' . $fileName;
                $storeId = $observer->getStore();
                $scopeLevel='stores';
            }
            if($observer->getWebsite()) {
                $scope=\Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
                $fileName = 'website/'.$observer->getWebsite().'/'.$fileName;
                $storeId = $observer->getWebsite();
                $scopeLevel='websites';
            }

            // Set Variables
            $jsContent = "let fonts_path='/".\Magento\Framework\App\Filesystem\DirectoryList::MEDIA . '/plazox/fonts'."';";

            if($this->scopeConfig->getValue('plazox/general/magento_theme',$scope,$storeId)){
                $jsContent .= PHP_EOL."let magento_theme='".$this->scopeConfig->getValue('plazox/general/magento_theme',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let magento_theme='".$this->scopeConfig->getValue('plazox/general/magento_theme')."';";
            }

            if($this->scopeConfig->getValue('plazox/bar_top_menu/show',$scope,$storeId)){
                $jsContent .= PHP_EOL."let bar_top_menu_show='".$this->scopeConfig->getValue('plazox/bar_top_menu/show_bar_top_menu',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let bar_top_menu_show='".$this->scopeConfig->getValue('plazox/bar_top_menu/show_bar_top_menu')."';";
            }

            if($this->scopeConfig->getValue('plazox/bar_top_menu/show_icon',$scope,$storeId)){
                $jsContent .= PHP_EOL."let bar_top_menu_show_icon='".$this->scopeConfig->getValue('plazox/bar_top_menu/show_icon',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let bar_top_menu_show_icon='".$this->scopeConfig->getValue('plazox/bar_top_menu/show_icon')."';";
            }

            if($this->scopeConfig->getValue('plazox/bar_top_menu/text_message',$scope,$storeId)){
                $jsContent .= PHP_EOL."let bar_top_menu_text_message='".$this->scopeConfig->getValue('plazox/bar_top_menu/text_message',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let bar_top_menu_text_message='".$this->scopeConfig->getValue('plazox/bar_top_menu/text_message')."';";
            }

            if($this->scopeConfig->getValue('plazox/bar_top_menu/style',$scope,$storeId)){
                $jsContent .= PHP_EOL."let bar_top_menu_style='".$this->scopeConfig->getValue('plazox/bar_top_menu/style',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let bar_top_menu_style='".$this->scopeConfig->getValue('plazox/bar_top_menu/style')."';";
            }

            if($this->scopeConfig->getValue('plazox/product_page/show_product_page',$scope,$storeId)){
                $jsContent .= PHP_EOL."let product_page_show='".$this->scopeConfig->getValue('plazox/product_page/show_product_page',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let product_page_show='".$this->scopeConfig->getValue('plazox/product_page/show_product_page')."';";
            }

            if($this->scopeConfig->getValue('plazox/product_page/show_banner',$scope,$storeId)){
                $jsContent .= PHP_EOL."let product_page_show_banner='".$this->scopeConfig->getValue('plazox/product_page/show_banner',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let product_page_show_banner='".$this->scopeConfig->getValue('plazox/product_page/show_banner')."';";
            }

            if($this->scopeConfig->getValue('plazox/floating_banner/visible',$scope,$storeId)){
                $jsContent .= PHP_EOL."let floating_banner_visible='".$this->scopeConfig->getValue('plazox/floating_banner/visible',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let floating_banner_visible='".$this->scopeConfig->getValue('plazox/floating_banner/visible')."';";
            }

            if($this->scopeConfig->getValue('plazox/floating_banner/position',$scope,$storeId)){
                $jsContent .= PHP_EOL."let floating_banner_position='".$this->scopeConfig->getValue('plazox/floating_banner/position',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let floating_banner_position='".$this->scopeConfig->getValue('plazox/floating_banner/position')."';";
            }

            if($this->scopeConfig->getValue('plazox/floating_banner/design',$scope,$storeId)){
                $jsContent .= PHP_EOL."let floating_banner_design='".$this->scopeConfig->getValue('plazox/floating_banner/design',$scope,$storeId)."';";
            }else{
                $jsContent .= PHP_EOL."let floating_banner_design='".$this->scopeConfig->getValue('plazox/floating_banner/design')."';";
            }

            // Incluye traducciones
            $translationJsContent = $this->getTranslationsFileContent();
            $jsContent.= PHP_EOL . $translationJsContent;

            // Incluye contenido base
            $baseJsContent = $this->getBaseFileContent();
            $jsContent.= PHP_EOL . $baseJsContent;

            //Si no hay cambios coge la ruta default
            if($this->isConfigInherited($scope, $storeId)){
                // Guarda la url del archivo JS
                $this->saveConfigValue('plazox/config/fileurl', $fileName, $scopeLevel, $storeId);
            }else{
                // Guarda la url del archivo DEFAULT JS
                $this->saveConfigValue('plazox/config/fileurl', 'plazox.js', $scopeLevel, $storeId);
            }

            // Ruta completa para el archivo JavaScript en pub/media
            $mediaDirectory = $this->filesystem->getDirectoryWrite(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA);
            $filePath = $mediaDirectory->getAbsolutePath('plazox/'.$fileName);

            // Guarda el contenido en el archivo JavaScript
            $mediaDirectory->writeFile('plazox/'.$fileName, $jsContent);
        }

        // Registrar mensaje de éxito
        if($baseJsContent!=null) {
            $this->messageManager->addSuccessMessage(__('¡The plazox configuration file has been generated successfully.!'));
        }else{
            $this->messageManager->addErrorMessage(__('¡Error generating the plazox configuration file'));
        }

        return $this;
    }

    /**
     * Obtiene el contenido del archivo basado en la ruta del archivo
     *
     * @param string $filePath
     * @return string
     */
    protected function getBaseFileContent()
    {
        // Obtener la ruta al archivo actual
        $currentDir = dirname(__FILE__);

        // Construir la ruta al archivo dos niveles por debajo del directorio actual
        $targetFilePath = $currentDir . '/../view/frontend/web/js/plazox-base.js';

        // Verificar si el archivo existe y leer su contenido
        if (file_exists($targetFilePath)) {
            $content = file_get_contents($targetFilePath);

            return $content;
        } else {
            return null;
        }
    }

    protected function getTranslationsFileContent()
    {
        // Obtener la ruta al archivo actual
        $currentDir = dirname(__FILE__);

        // Construir la ruta al archivo dos niveles por debajo del directorio actual
        $targetFilePath = $currentDir . '/../view/frontend/web/js/plazox-languages.js';

        // Verificar si el archivo existe y leer su contenido
        if (file_exists($targetFilePath)) {
            $content = file_get_contents($targetFilePath);

            return $content;
        } else {
            return null;
        }
    }

    /**
     * @param $path
     * @param $value
     * @param $scope
     * @param $scopeId
     * @return void
     */
    public function saveConfigValue($path, $value, $scope, $scopeId)
    {
        $this->configResource->saveConfig($path, $value, $scope, $scopeId);
    }

    /**
     * @return void
     */
    protected function copyFonts()
    {
        // Directorio de origen
        $sourceDir = dirname(__FILE__) . '/../view/frontend/web/fonts';

        // Directorio de destino
        $destDir = \Magento\Framework\App\Filesystem\DirectoryList::MEDIA . '/plazox/fonts';

        // Verificar si el directorio de destino existe, si no, crearlo
        if (!is_dir($destDir)) {
            mkdir($destDir, 0775, true);
        }

        // Obtener todos los archivos en el directorio de origen
        $files = scandir($sourceDir);

        foreach ($files as $file) {
            if ($file !== '.' && $file !== '..') {
                $sourceFilePath = $sourceDir . '/' . $file;
                $destFilePath = $destDir . '/' . $file;

                // Copiar archivo si es un archivo
                if (is_file($sourceFilePath)) {
                    copy($sourceFilePath, $destFilePath);
                }
            }
        }
    }

    private function isConfigInherited($scope, $storeId)
    {
        $tableName = 'core_config_data';
        $connection = $this->resource->getConnection();

        $select = $connection->select()
            ->from($tableName, ['*'])
            ->where('path LIKE ?', 'plazox/%')
            ->where('scope = ?', $scope.'s')
            ->where('scope_id = ?', $storeId)
            ->where('path <> ?', 'plazox/config/fileurl'); // Exclude specific path

        $result = $connection->fetchAll($select);

        if(!empty($result)){
            return true;
        }

        return false;
    }
}
