<?php
namespace Sistemapay\Plazox\Setup\Patch\Data;

use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\App\Config\Storage\WriterInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\Filesystem;
use Magento\Config\Model\ResourceModel\Config;

class ActivateModule implements DataPatchInterface
{
    private $moduleDataSetup;
    private $configWriter;
    protected $logger;
    protected $filesystem;
    protected $configResource;

    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        WriterInterface $configWriter,
        LoggerInterface $logger,
        Filesystem $filesystem,
        Config $configResource
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->configWriter = $configWriter;
        $this->logger = $logger;
        $this->filesystem = $filesystem;
        $this->configResource = $configResource;
    }

    public function apply()
    {
        $this->moduleDataSetup->startSetup();

        $config = [
            'plazox/general/magento_theme' => 'Luma',
            'plazox/bar_top_menu/show_bar_top_menu' => '1',
            'plazox/bar_top_menu/show_icon' => '1',
            'plazox/bar_top_menu/text_message' => 'Divide el pago con plazox',
            'plazox/bar_top_menu/style' => 'Light',
            'plazox/product_page/show_product_page' => '1',
            'plazox/product_page/show_banner' => 'Yes',
            'plazox/floating_banner/visible' => '1',
            'plazox/floating_banner/position' => 'Center',
            'plazox/floating_banner/design' => 'Option 1'
        ];

        foreach ($config as $path => $value) {
            $this->configWriter->save($path, $value);
        }


        $fileName='plazox.js';

        // Contenido del archivo JavaScript
        // Set Variables
        $jsContent = "let fonts_path='/".\Magento\Framework\App\Filesystem\DirectoryList::MEDIA . '/plazox/fonts'."';";
        $jsContent .= PHP_EOL."let magento_theme='".$config['plazox/general/magento_theme']."';";
        $jsContent .= PHP_EOL."let bar_top_menu_show='".$config['plazox/bar_top_menu/show_bar_top_menu']."';";
        $jsContent .= PHP_EOL."let bar_top_menu_show_icon='".$config['plazox/bar_top_menu/show_icon']."';";
        $jsContent .= PHP_EOL."let bar_top_menu_text_message='".$config['plazox/bar_top_menu/text_message']."';";
        $jsContent .= PHP_EOL."let bar_top_menu_style='".$config['plazox/bar_top_menu/style']."';";
        $jsContent .= PHP_EOL."let product_page_show='".$config['plazox/product_page/show_product_page']."';";
        $jsContent .= PHP_EOL."let product_page_show_banner='".$config['plazox/product_page/show_banner']."';";
        $jsContent .= PHP_EOL."let floating_banner_visible='".$config['plazox/floating_banner/visible']."';";
        $jsContent .= PHP_EOL."let floating_banner_position='".$config['plazox/floating_banner/position']."';";
        $jsContent .= PHP_EOL."let floating_banner_design='".$config['plazox/floating_banner/design']."';";
        $jsContent .= PHP_EOL;

        // Incluye traducciones
        $translationJsContent = $this->getTranslationsFileContent();
        $jsContent.= PHP_EOL . $translationJsContent;

        // Incluye contenido base
        $baseJsContent = $this->getBaseFileContent();
        $jsContent.= PHP_EOL . $baseJsContent; // Agregar una nueva línea*/


        // Guarda la url del archivo JS
        $this->saveConfigValue('plazox/config/fileurl', $fileName, 'default', 0);

        // Ruta completa para el archivo JavaScript en pub/media
        $mediaDirectory = $this->filesystem->getDirectoryWrite(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA);
        $filePath = $mediaDirectory->getAbsolutePath('plazox/'.$fileName);

        // Guarda el contenido en el archivo JavaScript
        $mediaDirectory->writeFile('plazox/'.$fileName, $jsContent);

        // Copia las fuentes a Media
        $this->copyFonts();


        $this->moduleDataSetup->endSetup();
    }

    public static function getDependencies()
    {
        return [];
    }

    public function getAliases()
    {
        return [];
    }

/**
* Obtiene el contenido del archivo basado en la ruta del archivo
*
* @param string $filePath
* @return string
*/
    protected function getBaseFileContent()
    {
        // Obtener la ruta al archivo actual
        $currentDir = dirname(__FILE__);

        // Construir la ruta al archivo dos niveles por debajo del directorio actual
        $targetFilePath = $currentDir . '/../../../view/frontend/web/js/plazox-base.js';

        // Verificar si el archivo existe y leer su contenido
        if (file_exists($targetFilePath)) {
            $content = file_get_contents($targetFilePath);

            return $content;
        } else {
            return null;
        }
    }

    protected function getTranslationsFileContent()
    {
        // Obtener la ruta al archivo actual
        $currentDir = dirname(__FILE__);

        // Construir la ruta al archivo dos niveles por debajo del directorio actual
        $targetFilePath = $currentDir . '/../../../view/frontend/web/js/plazox-languages.js';

        // Verificar si el archivo existe y leer su contenido
        if (file_exists($targetFilePath)) {
            $content = file_get_contents($targetFilePath);

            return $content;
        } else {
            return null;
        }
    }

    /**
     * @param $path
     * @param $value
     * @param $scope
     * @param $scopeId
     * @return void
     */
    public function saveConfigValue($path, $value, $scope, $scopeId)
    {
        $this->configResource->saveConfig($path, $value, $scope, $scopeId);
    }

    /**
     * @return void
     */
    protected function copyFonts()
    {
        // Directorio de origen
        $sourceDir = dirname(__FILE__) . '/../../../view/frontend/web/fonts';

        // Directorio de destino
        $destDir = \Magento\Framework\App\Filesystem\DirectoryList::MEDIA . '/plazox/fonts';

        // Verificar si el directorio de destino existe, si no, crearlo
        if (!is_dir($destDir)) {
            mkdir($destDir, 0775, true);
        }

        // Obtener todos los archivos en el directorio de origen
        $files = scandir($sourceDir);

        foreach ($files as $file) {
            if ($file !== '.' && $file !== '..') {
                $sourceFilePath = $sourceDir . '/' . $file;
                $destFilePath = $destDir . '/' . $file;

                // Copiar archivo si es un archivo
                if (is_file($sourceFilePath)) {
                    copy($sourceFilePath, $destFilePath);
                }
            }
        }
    }
}
